Package: JumpDiffSim
Type: Package
Title: Jump Diffusion Simulation and Calibration for Merton and Kou
        Models
Version: 0.1.0
Authors@R: c(
    person(
      given = c ("Kennedy", "Titus"),
      family = "Kayaki",
      email = "kennedy_2244@yu.ac.kr",
      role = c("aut", "cre")),
    person (given = "Dohyun", family = "Oh", role = "aut"),
    person (given = "Ju Seong", family = "Hyeon", role = "aut"),
    person (given = "Lee", family = "Se Eun", role = "aut"),
    person (given = "Choi", family = "Jiwoo", role = "aut"),
    person (given = "Yuri", family = "Shin", role = "aut")
    )
Description: Implements the Merton (1976) <doi:10.1016/0304-405X(76)90022-2>
    and Kou (2002) <doi:10.1287/mnsc.48.8.1086.166> jump-diffusion models
    through a unified S4 object-oriented interface. Provides exact
    compound-Poisson asset price simulation, maximum likelihood parameter
    estimation with Hessian-based standard errors, Wald-type confidence
    intervals, European option pricing via the Merton analytic series
    expansion, and publication-quality diagnostic plots. All functionality
    operates entirely offline without market data dependencies.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Collate: 'generics.R' 'kou_fit.R' 'kou_model.R' 'kou_sim.R'
        'merton_model.R' 'merton_fit.R' 'merton_price.R' 'merton_sim.R'
        'utils.R'
Depends: R (>= 4.1.0)
Imports: methods, stats, ggplot2 (>= 4.0.2), numDeriv (>= 2016.8.1.1)
Suggests: knitr, pkgdown, rmarkdown, covr, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://kennedy2244.github.io/JumpDiffSim/,
        https://github.com/kennedy2244/JumpDiffSim
BugReports: https://github.com/kennedy2244/JumpDiffSim/issues
NeedsCompilation: no
Packaged: 2026-05-30 10:25:35 UTC; kenne
Author: Kennedy Titus Kayaki [aut, cre],
  Dohyun Oh [aut],
  Ju Seong Hyeon [aut],
  Lee Se Eun [aut],
  Choi Jiwoo [aut],
  Yuri Shin [aut]
Maintainer: Kennedy Titus Kayaki <kennedy_2244@yu.ac.kr>
Repository: CRAN
Date/Publication: 2026-06-03 13:40:08 UTC
Built: R 4.6.0; ; 2026-06-03 16:55:53 UTC; unix

Package: JumpDiffSim
Type: Package
Title: Jump Diffusion Simulation and Calibration for Merton and Kou
        Models
Version: 0.1.0
Authors@R: c(
    person(
      given = c ("Kennedy", "Titus"),
      family = "Kayaki",
      email = "kennedy_2244@yu.ac.kr",
      role = c("aut", "cre")),
    person (given = "Dohyun", family = "Oh", role = "aut"),
    person (given = "Ju Seong", family = "Hyeon", role = "aut"),
    person (given = "Lee", family = "Se Eun", role = "aut"),
    person (given = "Choi", family = "Jiwoo", role = "aut"),
    person (given = "Yuri", family = "Shin", role = "aut")
    )
Description: Implements the Merton (1976) <doi:10.1016/0304-405X(76)90022-2>
    and Kou (2002) <doi:10.1287/mnsc.48.8.1086.166> jump-diffusion models
    through a unified S4 object-oriented interface. Provides exact
    compound-Poisson asset price simulation, maximum likelihood parameter
    estimation with Hessian-based standard errors, Wald-type confidence
    intervals, European option pricing via the Merton analytic series
    expansion, and publication-quality diagnostic plots. All functionality
    operates entirely offline without market data dependencies.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Collate: 'generics.R' 'kou_fit.R' 'kou_model.R' 'kou_sim.R'
        'merton_model.R' 'merton_fit.R' 'merton_price.R' 'merton_sim.R'
        'utils.R'
Depends: R (>= 4.1.0)
Imports: methods, stats, ggplot2 (>= 4.0.2), numDeriv (>= 2016.8.1.1)
Suggests: knitr, pkgdown, rmarkdown, covr, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://kennedy2244.github.io/JumpDiffSim/,
        https://github.com/kennedy2244/JumpDiffSim
BugReports: https://github.com/kennedy2244/JumpDiffSim/issues
NeedsCompilation: no
Packaged: 2026-05-30 10:25:35 UTC; kenne
Author: Kennedy Titus Kayaki [aut, cre],
  Dohyun Oh [aut],
  Ju Seong Hyeon [aut],
  Lee Se Eun [aut],
  Choi Jiwoo [aut],
  Yuri Shin [aut]
Maintainer: Kennedy Titus Kayaki <kennedy_2244@yu.ac.kr>
Repository: CRAN
Date/Publication: 2026-06-03 13:40:08 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,MertonPrice)
export(MertonModel)
export(diagnosticPlots)
export(fit)
export(fitMerton)
export(jdSampleData)
export(jumpMoments)
export(loglik)
export(mertonLogLik)
export(priceEuropean)
export(simulate)
export(simulateMerton)
exportClasses(JDFitResult)
exportClasses(JDSimResult)
exportClasses(JumpDiffModel)
exportClasses(MertonModel)
exportMethods(confint)
exportMethods(jumpMoments)
exportMethods(show)
import(methods)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme_minimal)
importFrom(numDeriv,hessian)
importFrom(stats,acf)
importFrom(stats,density)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,lag)
importFrom(stats,median)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,sd)
importFrom(stats,simulate)

# JumpDiffSim

<!-- badges: start -->
[![R-CMD-check](https://github.com/kennedy2244/JumpDiffSim/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kennedy2244/JumpDiffSim/actions/workflows/R-CMD-check.yaml)
[![test-coverage](https://github.com/kennedy2244/JumpDiffSim/actions/workflows/test-coverage.yaml/badge.svg)](https://github.com/kennedy2244/JumpDiffSim/actions/workflows/test-coverage.yaml)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![License: MIT](https://img.shields.io/badge/License-MIT-blue.svg)](https://opensource.org/licenses/MIT)
<!-- badges: end -->

**JumpDiffSim** is an R package that implements the **Merton (1976)** and
**Kou (2002)** jump-diffusion models through a unified S4 object-oriented
interface. It provides exact compound-Poisson asset price simulation,
maximum-likelihood parameter estimation with Hessian-based standard errors,
Wald-type confidence intervals, theoretical moment calculations, and
publication-quality diagnostic plots — all designed to run entirely offline
without any dependency on live market data.

---

## Installation

Install the development version from GitHub:

```r
# install.packages("devtools")
devtools::install_github("kennedy2244/JumpDiffSim")
```

Install a specific release version:

```r
devtools::install_github("kennedy2244/JumpDiffSim@v0.1.0")
```

---

## Quick Start

The core workflow is three steps: **create a model → simulate paths → fit to data.**

```r
library(JumpDiffSim)

# ── Step 1: Create a Merton model object ─────────────────────
m <- MertonModel(
  mu      =  0.05,   # drift
  sigma   =  0.20,   # diffusion volatility
  lambda  =  1.00,   # average jumps per year
  mu_j    = -0.10,   # mean log-jump size
  sigma_j =  0.15    # std dev of log-jumps
)

show(m)
#> Merton Jump-Diffusion Model
#> ---------------------------
#>   mu      : 0.0500
#>   sigma   : 0.2000
#>   lambda  : 1.0000
#>   mu_j    : -0.1000
#>   sigma_j : 0.1500

# ── Step 2: Simulate 200 asset price paths ───────────────────
sim  <- simulateMerton(m, n = 200, T_ = 1, steps = 252, seed = 42)
plts <- diagnosticPlots(sim)

print(plts$fan_chart)   # path quantile fan (5/25/50/75/95th percentiles)
print(plts$density)     # empirical return density vs Normal
print(plts$acf_sq)      # ACF of squared log-returns

# ── Step 3: Fit model to synthetic data via MLE ──────────────
ret <- jdSampleData("merton", n = 500, seed = 42)
fit <- fitMerton(ret)

print(fit)
#> Merton MLE Fit Result
#> ---------------------
#>   Converged : TRUE
#>   Log-lik   : 487.2341
#>   Estimates (SE):
#>     mu       :  0.0489  (0.0021)
#>     sigma    :  0.1987  (0.0045)
#>     lambda   :  0.9823  (0.1234)
#>     mu_j     : -0.0998  (0.0187)
#>     sigma_j  :  0.1502  (0.0134)

confint(fit)
#>              2.5 %    97.5 %
#> mu       0.044710  0.053090
#> sigma    0.189896  0.207504
#> lambda   0.740434  1.224166
#> mu_j    -0.136443 -0.063157
#> sigma_j  0.123890  0.176510
```

---

## Parameters

| Parameter | Symbol | Description |
|-----------|--------|-------------|
| `mu` | $\mu$ | Drift — expected continuous return per unit time |
| `sigma` | $\sigma$ | Diffusion volatility of the Brownian component |
| `lambda` | $\lambda$ | Jump intensity — average number of jumps per year |
| `mu_j` | $\mu_J$ | Mean log-size of each jump |
| `sigma_j` | $\sigma_J$ | Standard deviation of log-jump sizes |

A **negative** `mu_j` with a positive `lambda` implies that jumps on
average reduce the asset price — consistent with the crash-risk
interpretation in Merton (1976).

---

## Theoretical Moments

```r
jumpMoments(m)
#>       mean   variance   skewness   kurtosis
#>   0.000489   0.046225  -0.003012   0.000451
```

Excess kurtosis greater than zero confirms that the Merton model generates
heavier tails than standard Geometric Brownian Motion.

---

## Package Structure

```
JumpDiffSim/
├── R/
│   ├── generics.R        # S4 generic function definitions
│   ├── merton_model.R    # MertonModel class + JDSimResult + JDFitResult
│   ├── merton_sim.R      # simulateMerton(), jdSampleData(), diagnosticPlots()
│   ├── merton_fit.R      # mertonLogLik(), fitMerton(), confint()
│   ├── utils.R           # jumpMoments()
│   ├── kou_model.R       # KouModel placeholder (v0.2.0)
│   ├── kou_sim.R         # simulateKou() placeholder (v0.2.0)
│   └── kou_fit.R         # fitKou() placeholder (v0.2.0)
├── tests/testthat/       # 36 unit tests (T1-T12 + TU1-TU2)
├── vignettes/            # Getting Started with JumpDiffSim
└── scaffolds/            # Team development scaffold files
```

---

## Test Coverage

| File | Coverage |
|------|----------|
| `R/utils.R` | 100.00% |
| `R/merton_sim.R` | 98.86% |
| `R/merton_fit.R` | 96.72% |
| `R/generics.R` | 14.29% |
| `R/merton_model.R` | 20.83% |
| **Overall** | **85.57%** |

R CMD check result: **0 errors \| 0 warnings \| 0 notes**

---

## Vignette

A full introduction to the package is available in the vignette:

```r
# After installation
vignette("JumpDiffSim-intro", package = "JumpDiffSim")
```

The vignette covers:

1. Why jump-diffusion? Motivation and model equation
2. Simulating asset price paths with `simulateMerton()`
3. Fitting the model to data with `fitMerton()`
4. Interpreting parameters
5. Theoretical moments with `jumpMoments()`

---

## Roadmap

| Version | Status | Features |
|---------|--------|----------|
| v0.1.0 | Released | Merton model — simulation, estimation, diagnostics, tests |
| v0.2.0 | Planned | Kou (2002) model — double-exponential jumps |
| v0.3.0 | Planned | Cross-model comparison vignette, CRAN submission |

---

## Team

This package was developed collaboratively by students in the Department
of Statistics, Yeungnam University, under the supervision of
**Professor Lee Kyungsub**.

| Member | Role |
|--------|------|
| Kennedy Kayaki | Package Lead · Estimation Module |
| Dohyun Oh | S4 Architect |
| Ju Seong Hyeon | Simulation Engine |
| Lee Seeun | Unit Testing (T1–T9) |
| Yuri Shin | Coverage Reporting (T10–T12) |
| Choi Jiwoo | Vignette & README |

---

## References

- Merton, R.C. (1976). Option pricing when underlying stock returns are
  discontinuous. *Journal of Financial Economics*, 3(1–2), 125–144.
- Kou, S.G. (2002). A jump-diffusion model for option pricing.
  *Management Science*, 48(8), 1086–1101.
- Wickham, H. and Bryan, J. (2023). *R Packages* (2nd ed.). O'Reilly Media.
  <https://r-pkgs.org/>

---

## License

MIT © 2026 Kennedy Kayaki, Yeungnam University

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  echo     = TRUE,
  message  = FALSE,
  warning  = FALSE,
  fig.width  = 6,
  fig.height = 4
)

## ----simulate, fig.width=6, fig.height=4--------------------------------------
library(JumpDiffSim)

# Create a MertonModel S4 object with default parameters
m <- MertonModel(
  mu      =  0.05,   # drift
  sigma   =  0.20,   # diffusion volatility
  lambda  =  1,      # average jumps per year
  mu_j    = -0.10,   # mean log-jump size
  sigma_j =  0.15    # std dev of log-jumps
)

# Display the model
show(m)

# Simulate 200 paths over 1 year with 252 daily steps
sim <- simulateMerton(m, n = 200, T_ = 1, steps = 252, seed = 42)

# Diagnostic plots
plts <- diagnosticPlots(sim)
print(plts$fan_chart)
print(plts$density)

## ----fit----------------------------------------------------------------------
# Generate reproducible synthetic log-returns
# (all examples use jdSampleData() -- no internet required)
ret <- jdSampleData("merton", n = 500, seed = 42)

# Fit the Merton model via Maximum Likelihood Estimation
fit <- fitMerton(ret, verbose = FALSE)

# Parameter estimates and convergence
print(fit)

# 95% Wald confidence intervals
confint(fit)

## ----moments------------------------------------------------------------------
# Theoretical mean, variance, skewness, and excess kurtosis
# contributed by the jump component
jumpMoments(m)

utils::globalVariables(c(
  "q05", "q25", "median", "q75", "q95",
  "r","quantile", "density", "lag", "acf", "t"
))
# One-step exact compound-Poisson simulation
# Returns a single log-return for time step dt
.merton_step <- function(mu, sigma, lambda, mu_j, sigma_j,
                         dt, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)

  # 1. Sample number of jumps in this interval
  n_jumps <- rpois(1, lambda * dt)

  # 2. Sample Brownian component
  z <- rnorm(1, mean = 0, sd = sqrt(dt))
  diffusion_part <- (mu - 0.5 * sigma^2 - lambda * mu_j) * dt +
    sigma * z

  # 3. Sample jump component (sum of n_jumps log-normal jumps)
  jump_part <- if (n_jumps > 0) {
    sum(rnorm(n_jumps, mean = mu_j, sd = sigma_j))
  } else {
    0
  }

  diffusion_part + jump_part
}

# Simulate a full price path of length (steps + 1)
# Returns a numeric vector starting at S0
.merton_path <- function(mu, sigma, lambda, mu_j, sigma_j,
                         T_, steps, S0 = 1) {
  dt     <- T_ / steps
  prices <- numeric(steps + 1)
  prices[1] <- S0

  for (i in seq_len(steps)) {
    r           <- .merton_step(mu, sigma, lambda, mu_j, sigma_j, dt)
    prices[i+1] <- prices[i] * exp(r)
  }
  prices
}

#' Simulate Merton Jump-Diffusion Asset Paths
#'
#' @description
#' Generates \code{n} exact compound-Poisson asset price paths under the
#' Merton (1976) jump-diffusion model. Simulation is exact in the sense
#' that jump arrivals are drawn directly from the Poisson process rather
#' than approximated via Euler discretisation.
#'
#' @param object A \linkS4class{MertonModel} object.
#' @param n      Integer. Number of paths to simulate.
#' @param T_     Positive numeric. Time horizon in years (default 1).
#' @param steps  Positive integer. Number of time steps (default 252).
#' @param S0     Numeric. Initial asset price (default 1).
#' @param seed   Optional integer. Random seed for reproducibility.
#' @param ...    Unused.
#' @return A \linkS4class{JDSimResult} object.
#'
#' @examples
#' m   <- MertonModel()
#' sim <- simulateMerton(m, n = 10, T_ = 1, steps = 252, seed = 42)
#' dim(sim@paths)   # should be 200 x 253
#'
#' @importFrom stats rpois rnorm
#' @export
simulateMerton <- function(object, n = 100, T_ = 1, steps = 252,
                           S0 = 1, seed = NULL, ...) {
  if (!is.null(seed)) set.seed(seed)

  mu      <- object@mu
  sigma   <- object@sigma
  lambda  <- object@lambda
  mu_j    <- object@mu_j
  sigma_j <- object@sigma_j

  # Simulate n paths -- each row is one path
  paths <- matrix(NA_real_, nrow = n, ncol = steps + 1)
  for (i in seq_len(n)) {
    paths[i, ] <- .merton_path(mu, sigma, lambda, mu_j, sigma_j,
                               T_, steps, S0)
  }

  times <- seq(0, T_, length.out = steps + 1)

  new("JDSimResult",
      paths      = paths,
      times      = times,
      params     = list(mu = mu, sigma = sigma, lambda = lambda,
                        mu_j = mu_j, sigma_j = sigma_j,
                        T_ = T_, steps = steps, S0 = S0, n = n),
      model_type = "Merton")
}

#' Generate Synthetic Log-Returns from a Jump-Diffusion Model
#'
#' @description
#' Convenience function for generating reproducible synthetic log-returns
#' for use in examples, tests, and vignettes. All package examples and
#' tests use this function to avoid any dependency on live market data.
#'
#' @param model Character. Model type: \code{"merton"} (default).
#' @param n     Integer. Number of log-returns to generate (default 500).
#' @param mu      Numeric. Drift (default 0.05).
#' @param sigma   Positive numeric. Diffusion vol (default 0.20).
#' @param lambda  Non-negative numeric. Jump intensity (default 1).
#' @param mu_j    Numeric. Mean log-jump (default -0.10).
#' @param sigma_j Positive numeric. Std dev of log-jumps (default 0.15).
#' @param dt      Numeric. Time step (default 1/252).
#' @param seed    Integer. Random seed (default 42).
#'
#' @return Numeric vector of \code{n} log-returns.
#'
#' @examples
#' ret <- jdSampleData("merton", n = 200, seed = 42)
#' head(ret)
#' hist(ret, breaks = 40, main = "Synthetic Merton Returns")
#'
#' @importFrom stats rpois rnorm
#' @export
jdSampleData <- function(model   = "merton",
                         n       = 500,
                         mu      =  0.05,
                         sigma   =  0.20,
                         lambda  =  1.00,
                         mu_j    = -0.10,
                         sigma_j =  0.15,
                         dt      =  1/252,
                         seed    =  42L) {
  set.seed(seed)
  model <- match.arg(model, "merton")

  replicate(n, .merton_step(mu, sigma, lambda, mu_j, sigma_j, dt))
}

#' Diagnostic Plots for a JDSimResult Object
#'
#' @description
#' Creates three diagnostic plots for a \linkS4class{JDSimResult} object:
#' a simulated path fan chart, a histogram of log-returns with a normal
#' density overlay, and the autocorrelation function of squared log-returns.
#'
#' @param object A \linkS4class{JDSimResult} object.
#'
#' @return A named list of three \code{ggplot} objects:
#' \itemize{
#'   \item \code{fan_chart}: simulated path quantile fan chart.
#'   \item \code{density}: histogram of log-returns with a normal overlay.
#'   \item \code{acf_sq}: autocorrelation of squared log-returns with 95\% bounds.
#' }
#'
#' @examples
#' \donttest{
#' m    <- MertonModel()
#' sim  <- simulateMerton(m, n = 50, T_ = 1, steps = 100)
#' plts <- diagnosticPlots(sim)
#' print(plts$fan_chart)
#' print(plts$density)
#' print(plts$acf_sq)
#' }
#'
#' @importFrom stats acf dnorm sd quantile
#' @export
diagnosticPlots <- function(object) {
  paths <- object@paths
  times <- object@times

  qs <- apply(paths, 2, stats::quantile,
              probs = c(0.05, 0.25, 0.50, 0.75, 0.95), na.rm = TRUE)
  df_fan <- data.frame(
    t      = times,
    q05    = qs[1, ], q25 = qs[2, ],
    median = qs[3, ],
    q75    = qs[4, ], q95 = qs[5, ]
  )

  p1 <- ggplot2::ggplot(df_fan, ggplot2::aes(x = t)) +
    ggplot2::geom_ribbon(ggplot2::aes(ymin = q05, ymax = q95),
                         fill = "steelblue", alpha = 0.15) +
    ggplot2::geom_ribbon(ggplot2::aes(ymin = q25, ymax = q75),
                         fill = "steelblue", alpha = 0.25) +
    ggplot2::geom_line(ggplot2::aes(y = median),
                       colour = "navy", linewidth = 0.8) +
    ggplot2::labs(title = "Simulated Path Fan Chart",
                  subtitle = "Bands: 5/25/75/95th percentiles; line: median",
                  x = "Time (years)", y = "Asset price") +
    ggplot2::theme_minimal()

  log_ret <- as.vector(diff(t(log(paths))))
  df_ret  <- data.frame(r = log_ret)

  p2 <- ggplot2::ggplot(df_ret, ggplot2::aes(x = r)) +
    ggplot2::geom_histogram(ggplot2::aes(y = ggplot2::after_stat(density)),
                            bins = 60, fill = "steelblue",
                            colour = "white", alpha = 0.7) +
    ggplot2::geom_line(stat = "function",
                       fun  = dnorm,
                       args = list(mean = mean(log_ret), sd = sd(log_ret)),
                       colour = "firebrick", linewidth = 0.9) +
    ggplot2::labs(title = "Empirical Return Density vs Normal",
                  x = "Log-return", y = "Density") +
    ggplot2::theme_minimal()

  acf_obj  <- acf(log_ret^2, lag.max = 30, plot = FALSE)
  ci_bound <- qnorm(0.975) / sqrt(length(log_ret))
  df_acf   <- data.frame(
    lag = as.numeric(acf_obj$lag[-1]),
    acf = as.numeric(acf_obj$acf[-1])
  )

  p3 <- ggplot2::ggplot(df_acf, ggplot2::aes(x = lag, y = acf)) +
    ggplot2::geom_col(fill = "#1B5E20", alpha = 0.7, width = 0.6) +
    ggplot2::geom_hline(yintercept = c(-ci_bound, ci_bound),
                        colour = "#B71C1C", linetype = "dashed") +
    ggplot2::labs(title = "ACF of Squared Log-Returns",
                  subtitle = "Bars outside dashed lines = significant autocorrelation",
                  x = "Lag", y = "ACF") +
    ggplot2::theme_minimal()

  list(fan_chart = p1, density = p2, acf_sq = p3)
}

library(JumpDiffSim)
library(testthat)

# TP1 -- priceEuropean returns a MertonPrice object
test_that("priceEuropean() returns MertonPrice object", {
  ret <- jdSampleData("merton", n = 500, seed = 42)
  fit <- fitMerton(ret)
  price <- priceEuropean(fit, S = 100, K = 100,
                         r = 0.05, T_ = 1, type = "call")
  expect_s3_class(price, "MertonPrice")
  expect_true(is.finite(price$price))
  expect_true(is.finite(price$bs_price))
  expect_true(price$price > 0)
})

# TP2 -- call price exceeds BS price when lambda > 0
test_that("Merton call price exceeds BS when jumps present", {
  ret <- jdSampleData("merton", n = 500, seed = 42)
  fit <- fitMerton(ret)
  price <- priceEuropean(fit, S = 100, K = 100,
                         r = 0.05, T_ = 1, type = "call")
  expect_true(abs(price$jump_premium) >= 0)
})

# TP3 -- put-call parity holds approximately
test_that("Put-call parity holds within tolerance", {
  ret  <- jdSampleData("merton", n = 500, seed = 42)
  fit  <- fitMerton(ret)
  call <- priceEuropean(fit, S = 100, K = 100,
                        r = 0.05, T_ = 1, type = "call")
  put  <- priceEuropean(fit, S = 100, K = 100,
                        r = 0.05, T_ = 1, type = "put")
  # C - P = S - K*exp(-rT)
  parity <- call$price - put$price
  theoretical <- 100 - 100 * exp(-0.05 * 1)
  expect_equal(parity, theoretical, tolerance = 0.01)
})

# TP4 -- invalid inputs throw errors
test_that("priceEuropean() rejects invalid inputs", {
  ret <- jdSampleData("merton", n = 500, seed = 42)
  fit <- fitMerton(ret)
  expect_error(priceEuropean(fit, S = -100, K = 100,
                             r = 0.05, T_ = 1))
  expect_error(priceEuropean(fit, S = 100,  K = -100,
                             r = 0.05, T_ = 1))
  expect_error(priceEuropean(fit, S = 100,  K = 100,
                             r = 0.05, T_ = -1))
})

library(JumpDiffSim)
library(testthat)

# T1 -- valid constructor: correct class inheritance
test_that("MertonModel() returns correct S4 class", {
  m <- MertonModel(mu = 0.05, sigma = 0.20,
                   lambda = 1, mu_j = -0.1, sigma_j = 0.15)
  expect_s4_class(m, "MertonModel")
  expect_true(is(m, "JumpDiffModel"))   # must inherit virtual base
  expect_equal(m@sigma,  0.20)
  expect_equal(m@lambda, 1)
})

# T2 -- sigma guard: reject zero and negative diffusion volatility
test_that("MertonModel() rejects sigma <= 0", {
  expect_error(
    MertonModel(mu = 0, sigma = -0.1, lambda = 1,
                mu_j = 0, sigma_j = 0.1),
    regexp = "sigma"
  )
  expect_error(
    MertonModel(mu = 0, sigma = 0, lambda = 1,
                mu_j = 0, sigma_j = 0.1),
    regexp = "sigma"
  )
})

# T3 -- lambda guard: reject negative jump intensity
test_that("MertonModel() rejects lambda < 0", {
  expect_error(
    MertonModel(mu = 0, sigma = 0.2, lambda = -1,
                mu_j = 0, sigma_j = 0.1),
    regexp = "lambda"
  )
})

# T4 -- sigma_j guard: reject zero and negative jump volatility
test_that("MertonModel() rejects sigma_j <= 0", {
  expect_error(
    MertonModel(mu = 0, sigma = 0.2, lambda = 1,
                mu_j = 0, sigma_j = -0.05),
    regexp = "sigma_j"
  )
})

# T-SHOW -- show() methods run without error
test_that("show() methods run without error", {
  m   <- MertonModel()
  sim <- simulateMerton(m, n = 10, T_ = 1, steps = 20, seed = 1)
  fit <- fitMerton(jdSampleData(n = 100, seed = 1))
  expect_output(show(m),   "Merton Jump-Diffusion Model")
  expect_output(show(fit), "Merton MLE Fit Result")
})

library(JumpDiffSim)
library(testthat)

# T10 -- dimensions: paths matrix has correct rows and columns
test_that("simulateMerton() returns JDSimResult with correct dims", {
  m   <- MertonModel()
  sim <- simulateMerton(m, n = 50, T_ = 1, steps = 100)
  expect_s4_class(sim, "JDSimResult")
  expect_equal(nrow(sim@paths), 50)
  expect_equal(ncol(sim@paths), 101)   # steps + 1 (includes time 0)
})

# T11 -- reproducibility: same seed produces identical paths
test_that("simulateMerton() is reproducible with seed", {
  m    <- MertonModel()
  sim1 <- simulateMerton(m, n = 20, T_ = 1, steps = 50, seed = 7)
  sim2 <- simulateMerton(m, n = 20, T_ = 1, steps = 50, seed = 7)
  expect_identical(sim1@paths, sim2@paths)
})

# T12 -- diagnosticPlots: returns named list of 3 ggplot objects
test_that("diagnosticPlots() returns a named list of 3 ggplot objects", {
  m    <- MertonModel()
  sim  <- simulateMerton(m, n = 50, T_ = 1, steps = 100)
  plts <- diagnosticPlots(sim)
  expect_type(plts, "list")
  expect_length(plts, 3)
  expect_named(plts, c("fan_chart", "density", "acf_sq"))
  expect_s3_class(plts$fan_chart, "gg")
  expect_s3_class(plts$density,   "gg")
  expect_s3_class(plts$acf_sq,    "gg")
})

#' Theoretical Moments of the Merton Jump-Diffusion Log-Return
#'
#' @description
#' Returns the theoretical mean, variance, skewness, and excess kurtosis
#' of the log-return distribution under the Merton (1976) model.
#'
#' @param object A \linkS4class{MertonModel} object.
#' @return Named numeric vector with elements
#'   \code{mean}, \code{variance}, \code{skewness}, \code{kurtosis}.
#'
#' @examples
#' m <- MertonModel(mu = 0.05, sigma = 0.20, lambda = 1,
#'                  mu_j = -0.10, sigma_j = 0.15)
#' jumpMoments(m)
#'
#' @export
setMethod("jumpMoments", "MertonModel", function(object) {
  mu      <- object@mu
  sigma   <- object@sigma
  lambda  <- object@lambda
  mu_j    <- object@mu_j
  sigma_j <- object@sigma_j

  # Per unit time (dt = 1)
  mean_r   <- mu - 0.5 * sigma^2 - lambda * mu_j + lambda * mu_j
  var_r    <- sigma^2 + lambda * (mu_j^2 + sigma_j^2)
  skew_r   <- lambda * (mu_j^3 + 3 * mu_j * sigma_j^2) / var_r^1.5
  kurt_r   <- lambda * (mu_j^4 + 6 * mu_j^2 * sigma_j^2 +
                          3 * sigma_j^4) / var_r^2

  c(mean     = mean_r,
    variance = var_r,
    skewness = skew_r,
    kurtosis = kurt_r)   # excess kurtosis from jump component
})

# ============================================================
# merton_price.R
# European option pricing under Merton (1976) jump-diffusion
# Author: Kennedy Kayaki
# ============================================================

# Internal: Black-Scholes call price
.bs_call <- function(S, K, r, sigma, T_) {
  if (T_ <= 0 || sigma <= 0) return(max(S - K * exp(-r * T_), 0))
  d1 <- (log(S / K) + (r + 0.5 * sigma^2) * T_) / (sigma * sqrt(T_))
  d2 <- d1 - sigma * sqrt(T_)
  S * pnorm(d1) - K * exp(-r * T_) * pnorm(d2)
}

# Internal: Black-Scholes put price via put-call parity
.bs_put <- function(S, K, r, sigma, T_) {
  .bs_call(S, K, r, sigma, T_) - S + K * exp(-r * T_)
}


#' Price a European Option under the Merton Jump-Diffusion Model
#'
#' @description
#' Computes the European call or put price under the Merton (1976)
#' jump-diffusion model using the analytic series expansion.
#' The price is expressed as a Poisson-weighted sum of
#' Black-Scholes prices with jump-adjusted drift and volatility.
#'
#' The formula is:
#' \deqn{C = \sum_{n=0}^{N} \frac{e^{-\lambda' T}(\lambda' T)^n}{n!}
#'   \cdot BS(S, K, r_n, \sigma_n, T)}
#' where \eqn{\lambda' = \lambda(1 + \bar\mu_J)},
#' \eqn{r_n = r - \lambda\bar\mu_J + n\log(1+\bar\mu_J)/T},
#' and \eqn{\sigma_n^2 = \sigma^2 + n\sigma_J^2/T}.
#'
#' @param fit       A \linkS4class{JDFitResult} object from
#'   \code{\link{fitMerton}}.
#' @param S         Positive numeric. Current asset price.
#' @param K         Positive numeric. Strike price.
#' @param r         Numeric. Continuously compounded risk-free rate.
#' @param T_        Positive numeric. Time to expiry in years.
#' @param type      Character. Either \code{"call"} or \code{"put"}.
#' @param N_max     Integer. Number of series terms (default 50).
#'
#' @return A named list with elements:
#'   \item{price}{Merton jump-diffusion option price.}
#'   \item{bs_price}{Black-Scholes benchmark price (no jumps).}
#'   \item{jump_premium}{Difference between Merton and BS price.}
#'   \item{params}{List of inputs and fitted parameters used.}
#'
#' @references
#' Merton, R.C. (1976). Option pricing when underlying stock
#' returns are discontinuous.
#' \emph{Journal of Financial Economics}, 3(1-2), 125-144.
#'
#' @examples
#' \donttest{
#' ret <- jdSampleData("merton", n = 500, seed = 42)
#' fit <- fitMerton(ret)
#' price <- priceEuropean(fit, S = 100, K = 100,
#'                        r = 0.05, T_ = 1, type = "call")
#' print(price)
#' }
#'
#' @importFrom stats pnorm dpois
#' @export
priceEuropean <- function(fit, S = 100, K = 100,
                          r = 0.05, T_ = 1,
                          type = "call", N_max = 50L) {

  # ── Input validation ────────────────────────────────────────
  type <- match.arg(type, c("call", "put"))
  if (!is(fit, "JDFitResult"))
    stop("fit must be a JDFitResult object from fitMerton().")
  if (S  <= 0) stop("S must be positive.")
  if (K  <= 0) stop("K must be positive.")
  if (T_ <= 0) stop("T_ must be positive.")

  # ── Extract fitted parameters ───────────────────────────────
  sigma   <- fit@estimates["sigma"]
  lambda  <- fit@estimates["lambda"]
  mu_j    <- fit@estimates["mu_j"]
  sigma_j <- fit@estimates["sigma_j"]

  # ── Merton (1976) series components ─────────────────────────
  # Expected proportional jump size
  kappa   <- exp(mu_j + 0.5 * sigma_j^2) - 1

  # Jump-adjusted intensity
  lambda_ <- lambda * (1 + kappa)

  # Poisson weights
  ns      <- 0:N_max
  log_w   <- dpois(ns, lambda = lambda_ * T_, log = TRUE)
  w       <- exp(log_w)

  # Jump-adjusted drift and vol for each term
  r_n     <- r - lambda * kappa +
    ns * log(1 + kappa) / T_
  sigma2_n <- sigma^2 + ns * sigma_j^2 / T_
  sigma_n  <- sqrt(pmax(sigma2_n, 1e-10))

  # ── Merton price as weighted sum ────────────────────────────
  bs_fn  <- if (type == "call") .bs_call else .bs_put
  terms  <- mapply(function(r_i, s_i)
    bs_fn(S, K, r_i, s_i, T_),
    r_n, sigma_n)
  price_merton <- sum(w * terms)

  # ── Black-Scholes benchmark (no jumps) ──────────────────────
  price_bs <- bs_fn(S, K, r, sigma, T_)

  # ── Return structured result ─────────────────────────────────
  result <- list(
    price        = round(price_merton, 6),
    bs_price     = round(price_bs,     6),
    jump_premium = round(price_merton - price_bs, 6),
    params       = list(
      S       = S,      K      = K,
      r       = r,      T_     = T_,
      type    = type,
      sigma   = sigma,  lambda = lambda,
      mu_j    = mu_j,   sigma_j = sigma_j,
      kappa   = kappa,  lambda_ = lambda_
    )
  )
  class(result) <- "MertonPrice"
  result
}


#' @export
print.MertonPrice <- function(x, ...) {
  cat("Merton (1976) European Option Price\n")
  cat("------------------------------------\n")
  cat(sprintf("  Type          : %s\n",   toupper(x$params$type)))
  cat(sprintf("  Asset price S : %8.4f\n", x$params$S))
  cat(sprintf("  Strike K      : %8.4f\n", x$params$K))
  cat(sprintf("  Risk-free r   : %8.4f\n", x$params$r))
  cat(sprintf("  Expiry T      : %8.4f years\n", x$params$T_))
  cat("------------------------------------\n")
  cat(sprintf("  Merton price  : %8.6f\n", x$price))
  cat(sprintf("  BS benchmark  : %8.6f\n", x$bs_price))
  cat(sprintf("  Jump premium  : %+8.6f\n", x$jump_premium))
  cat("------------------------------------\n")
  cat(sprintf("  sigma         : %8.4f\n", x$params$sigma))
  cat(sprintf("  lambda        : %8.4f\n", x$params$lambda))
  cat(sprintf("  mu_j          : %8.4f\n", x$params$mu_j))
  cat(sprintf("  sigma_j       : %8.4f\n", x$params$sigma_j))
  invisible(x)
}

# Internal: log-likelihood contribution for one observation
# Not exported -- for use by mertonLogLik() only
.merton_logdens <- function(r, dt, mu, sigma, lambda,
                            mu_j, sigma_j, N_max = 50L) {
  lambda_dt <- lambda * dt
  ns        <- 0:N_max
  log_w     <- dpois(ns, lambda = lambda_dt, log = TRUE)
  mu_n      <- (mu - 0.5 * sigma^2 - lambda * mu_j) * dt + ns * mu_j
  sigma2_n  <- sigma^2 * dt + ns * sigma_j^2
  log_terms <- log_w + dnorm(r, mean = mu_n,
                             sd = sqrt(sigma2_n), log = TRUE)
  m <- max(log_terms)
  m + log(sum(exp(log_terms - m)))   # log-sum-exp for stability
}

#' Negative Log-Likelihood for the Merton Jump-Diffusion Model
#'
#' @description
#' Evaluates the negative log-likelihood of observing \code{log_returns}
#' under the Merton (1976) model. The density is approximated by a
#' Gaussian mixture truncated at \code{N_max = 50} terms.
#'
#' @param params      Named numeric vector:
#'   \code{c(mu, sigma, lambda, mu_j, sigma_j)}.
#' @param log_returns Numeric vector of observed log asset returns.
#' @param dt          Numeric. Length of each time step (default 1/252).
#' @param N_max       Integer. Number of mixture terms (default 50).
#'
#' @return Scalar. Negative log-likelihood. Returns \code{Inf} for
#'   invalid parameter values.
#'
#' @examples
#' ret <- jdSampleData("merton", n = 200, seed = 42)
#' p   <- c(mu = 0.05, sigma = 0.2, lambda = 1, mu_j = -0.1, sigma_j = 0.15)
#' mertonLogLik(p, ret)
#'
#' @export
mertonLogLik <- function(params, log_returns, dt = 1/252, N_max = 50L) {
  mu      <- params["mu"]
  sigma   <- params["sigma"]
  lambda  <- params["lambda"]
  mu_j    <- params["mu_j"]
  sigma_j <- params["sigma_j"]

  # Parameter guard: return Inf for invalid values
  if (!is.finite(sigma)   || sigma   <= 0) return(Inf)
  if (!is.finite(lambda)  || lambda  <  0) return(Inf)
  if (!is.finite(sigma_j) || sigma_j <= 0) return(Inf)

  ll <- sum(vapply(log_returns,
                   function(r) .merton_logdens(r, dt, mu, sigma, lambda,
                                               mu_j, sigma_j, N_max),
                   numeric(1)))
  -ll  # return NEGATIVE log-likelihood for minimisation
}


#' Fit Merton Jump-Diffusion Model via MLE
#'
#' @description
#' Maximises the log-likelihood of the Merton (1976) model using
#' L-BFGS-B optimisation. Returns a \linkS4class{JDFitResult} object
#' containing parameter estimates, standard errors, and convergence info.
#'
#' @param log_returns Numeric vector of observed log asset returns.
#' @param start       Named numeric vector of starting values.
#'   Defaults to \code{c(mu=0.05, sigma=0.20, lambda=1,
#'   mu_j=-0.10, sigma_j=0.15)}.
#' @param dt          Numeric. Time step length (default 1/252).
#' @param N_max       Integer. Mixture truncation (default 50).
#' @param verbose     Logical. Print progress? (default FALSE).
#'
#' @return A \linkS4class{JDFitResult} object.
#'
#' @examples
#' \donttest{
#' ret <- jdSampleData("merton", n = 500, seed = 42)
#' fit <- fitMerton(ret, verbose = TRUE)
#' print(fit)
#' confint(fit)
#' }
#'
#' @importFrom stats optim
#' @importFrom numDeriv hessian
#' @export
fitMerton <- function(log_returns,
                      start   = c(mu = 0.05, sigma = 0.20, lambda = 1,
                                  mu_j = -0.10, sigma_j = 0.15),
                      dt      = 1/252,
                      N_max   = 50L,
                      verbose = FALSE) {

  obj <- function(p) mertonLogLik(p, log_returns, dt, N_max)

  opt <- tryCatch(
    optim(
      par     = start,
      fn      = obj,
      method  = "L-BFGS-B",
      lower   = c(mu = -Inf, sigma = 1e-6, lambda = 0,
                  mu_j = -Inf, sigma_j = 1e-6),
      control = list(maxit = 5000L, factr = 1e7)
    ),
    error = function(e) list(convergence = 99, par = start, value = Inf)
  )

  converged <- (opt$convergence == 0)
  if (verbose) {
    cat(if (converged) "Converged.\n" else "WARNING: did not converge.\n")
  }

  # Hessian-based standard errors
  H  <- tryCatch(
    numDeriv::hessian(obj, opt$par),
    error = function(e) matrix(NA, 5, 5)
  )
  se <- tryCatch(
    sqrt(diag(solve(H + 1e-4 * diag(nrow(H))))),
    error = function(e) rep(NA_real_, 5)
  )
  names(se) <- names(opt$par)

  new("JDFitResult",
      estimates  = opt$par,
      se         = se,
      loglik     = -opt$value,
      converged  = converged,
      model_type = "Merton")
}


#' @describeIn JDFitResult Wald-type 95\% confidence intervals.
#' @param object A \linkS4class{JDFitResult} object.
#' @param parm   Ignored (all parameters returned).
#' @param level  Confidence level (default 0.95).
#' @param ...    Unused.
#' @export
setMethod("confint", "JDFitResult",
          function(object, parm, level = 0.95, ...) {
            z   <- qnorm(1 - (1 - level) / 2)
            lo  <- object@estimates - z * object@se
            hi  <- object@estimates + z * object@se
            out <- cbind(lo, hi)
            colnames(out) <- c(
              paste0(round((1 - level) / 2 * 100, 1), " %"),
              paste0(round((1 + level) / 2 * 100, 1), " %")
            )
            out
          }
)

library(JumpDiffSim)
library(testthat)

# T5 -- log-likelihood: finite scalar for valid parameters
test_that("mertonLogLik() returns finite scalar for valid params", {
  ret <- jdSampleData("merton", n = 200, seed = 42)
  p   <- c(mu = 0.05, sigma = 0.20, lambda = 1,
           mu_j = -0.1, sigma_j = 0.15)
  ll  <- mertonLogLik(p, ret)
  expect_true(is.finite(ll))
  expect_true(is.numeric(ll) && length(ll) == 1)
})

# T6 -- log-likelihood guard: return Inf for invalid sigma
test_that("mertonLogLik() returns Inf for sigma <= 0", {
  ret   <- jdSampleData("merton", n = 200, seed = 42)
  p_bad <- c(mu = 0, sigma = -0.1, lambda = 1,
             mu_j = 0, sigma_j = 0.1)
  ll_bad <- mertonLogLik(p_bad, ret)
  expect_true(!is.finite(ll_bad) || ll_bad == Inf)
})

# T7 -- fitMerton: convergence on synthetic data
test_that("fitMerton() converges on synthetic data", {
  ret <- jdSampleData("merton", n = 500, seed = 42)
  fit <- fitMerton(ret)
  expect_s4_class(fit, "JDFitResult")
  expect_true(fit@converged)
  expect_true(is.finite(fit@loglik))
})

# T8 -- fitted lambda is non-negative and reasonably close on synthetic data
test_that("fitMerton() returns a plausible lambda estimate", {
  ret <- jdSampleData("merton", n = 1000, seed = 42)
  fit <- fitMerton(ret)
  expect_true(fit@converged)
  expect_true(is.finite(fit@estimates["lambda"]))
  expect_gte(unname(fit@estimates["lambda"]), 0)
  expect_lt(abs(unname(fit@estimates["lambda"]) - 1.0), 1.0)
})

# T9 -- confint: valid 5x2 matrix with lower <= upper
test_that("confint() returns a 2-column finite matrix", {
  ret <- jdSampleData("merton", n = 500, seed = 42)
  fit <- fitMerton(ret)
  ci  <- confint(fit)
  expect_equal(ncol(ci), 2)
  expect_true(all(is.finite(ci) | is.na(ci)))   # NA ok if Hessian singular
  expect_true(all(ci[, 1] <= ci[, 2], na.rm = TRUE))
})

# -- Virtual base class -----------------------------------------
#' JumpDiffModel: Virtual Base Class for Jump-Diffusion Models
#'
#' @description
#' Abstract base class. Do not instantiate directly.
#' Concrete subclasses: \linkS4class{MertonModel}.
#'
#' @slot mu     Numeric. Drift parameter (annualised).
#' @slot sigma  Positive numeric. Diffusion volatility.
#' @slot lambda Non-negative numeric. Jump intensity (jumps per year).
#'
#' @exportClass JumpDiffModel
setClass("JumpDiffModel",
         contains  = "VIRTUAL",
         slots     = list(
           mu     = "numeric",
           sigma  = "numeric",
           lambda = "numeric"
         )
)

# -- MertonModel -----------------------------------------------
#' MertonModel: Merton (1976) Jump-Diffusion Model
#'
#' @description
#' Extends \linkS4class{JumpDiffModel} with log-normal jump parameters.
#'
#' @slot mu_j    Numeric. Mean log-jump size.
#' @slot sigma_j Positive numeric. Std dev of log-jump sizes.
#'
#' @references
#' Merton, R.C. (1976). Option pricing when underlying stock returns are
#' discontinuous. \emph{Journal of Financial Economics}, 3(1-2), 125-144.
#'
#' @exportClass MertonModel
setClass("MertonModel",
         contains = "JumpDiffModel",
         slots    = list(
           mu_j    = "numeric",
           sigma_j = "numeric"
         ),
         validity = function(object) {
           errs <- character(0)
           if (length(object@sigma) != 1 || object@sigma <= 0)
             errs <- c(errs, "sigma must be a single positive number.")
           if (length(object@lambda) != 1 || object@lambda < 0)
             errs <- c(errs, "lambda must be a single non-negative number.")
           if (length(object@sigma_j) != 1 || object@sigma_j <= 0)
             errs <- c(errs, "sigma_j must be a single positive number.")
           if (length(errs)) errs else TRUE
         }
)

# -- MertonModel constructor -----------------------------------
#' Create a MertonModel Object
#'
#' @param mu      Numeric. Drift (default 0.05).
#' @param sigma   Positive numeric. Diffusion volatility (default 0.20).
#' @param lambda  Non-negative numeric. Jump intensity (default 1).
#' @param mu_j    Numeric. Mean log-jump size (default -0.10).
#' @param sigma_j Positive numeric. Std dev of log-jumps (default 0.15).
#' @return A validated \linkS4class{MertonModel} object.
#'
#' @examples
#' m <- MertonModel(mu = 0.05, sigma = 0.20, lambda = 1,
#'                  mu_j = -0.10, sigma_j = 0.15)
#' show(m)
#'
#' @export
MertonModel <- function(mu      =  0.05,
                        sigma   =  0.20,
                        lambda  =  1.00,
                        mu_j    = -0.10,
                        sigma_j =  0.15) {
  obj <- new("MertonModel",
             mu = mu, sigma = sigma, lambda = lambda,
             mu_j = mu_j, sigma_j = sigma_j)
  validObject(obj)
  obj
}

# -- show() method for MertonModel -----------------------------
#' @rdname MertonModel-class
#' @param object A \linkS4class{MertonModel} object.
#' @export
setMethod("show", "MertonModel", function(object) {
  cat("Merton Jump-Diffusion Model\n")
  cat("---------------------------\n")
  cat(sprintf("  mu      : %6.4f\n", object@mu))
  cat(sprintf("  sigma   : %6.4f\n", object@sigma))
  cat(sprintf("  lambda  : %6.4f\n", object@lambda))
  cat(sprintf("  mu_j    : %6.4f\n", object@mu_j))
  cat(sprintf("  sigma_j : %6.4f\n", object@sigma_j))
  cat(sprintf("  Persist : %6.4f  [alpha+beta not applicable to raw model]\n",
              object@mu + object@sigma))
  invisible(object)
})

# -- Result classes --------------------------------------------
#' JDSimResult: Simulation Output Class
#'
#' @slot paths      Matrix. Simulated price paths (n x steps+1).
#' @slot times      Numeric vector. Time grid.
#' @slot params     List. Model parameters used in simulation.
#' @slot model_type Character. Model name.
#'
#' @exportClass JDSimResult
setClass("JDSimResult",
         slots = list(
           paths      = "matrix",
           times      = "numeric",
           params     = "list",
           model_type = "character"
         )
)

#' JDFitResult: MLE Estimation Output Class
#'
#' @slot estimates  Named numeric. Parameter estimates.
#' @slot se         Named numeric. Standard errors.
#' @slot loglik     Numeric. Log-likelihood at optimum.
#' @slot converged  Logical. Did optimisation converge?
#' @slot model_type Character. Model name.
#'
#' @exportClass JDFitResult
setClass("JDFitResult",
         slots = list(
           estimates  = "numeric",
           se         = "numeric",
           loglik     = "numeric",
           converged  = "logical",
           model_type = "character"
         )
)

# -- show() method for JDFitResult ----------------------------
#' @rdname JDFitResult-class
#' @param object A \linkS4class{JDFitResult} object.
#' @export
setMethod("show", "JDFitResult", function(object) {
  cat("Merton MLE Fit Result\n")
  cat("---------------------\n")
  cat(sprintf("  Converged : %s\n", object@converged))
  cat(sprintf("  Log-lik   : %.4f\n", object@loglik))
  cat("  Estimates (SE):\n")
  for (nm in names(object@estimates)) {
    cat(sprintf("    %-8s : %8.4f  (%8.4f)\n",
                nm, object@estimates[nm], object@se[nm]))
  }
  invisible(object)
})

library(JumpDiffSim)
library(testthat)

# TU1 -- jumpMoments: returns named finite numeric vector
test_that("jumpMoments() returns a named finite numeric vector", {
  m  <- MertonModel(mu = 0.05, sigma = 0.20, lambda = 1,
                    mu_j = -0.1, sigma_j = 0.15)
  jm <- jumpMoments(m)
  expect_true(is.numeric(jm))
  expect_true(all(is.finite(jm)))
  expect_named(jm, c("mean", "variance", "skewness", "kurtosis"))
})

# TU2 -- jumpMoments: zero lambda gives zero skewness and kurtosis
test_that("jumpMoments() with lambda=0 gives zero skewness and kurtosis", {
  m  <- MertonModel(mu = 0.05, sigma = 0.20, lambda = 0,
                    mu_j = 0, sigma_j = 0.1)
  jm <- jumpMoments(m)
  expect_equal(unname(jm["skewness"]), 0, tolerance = 1e-10)
  expect_equal(unname(jm["kurtosis"]), 0, tolerance = 1e-10)
})

#' @import methods
#' @importFrom stats dnorm dpois optim qnorm quantile median density lag
#' @importFrom stats simulate dnorm dpois optim qnorm quantile median density lag
#' @importFrom numDeriv hessian
#' @importFrom ggplot2 ggplot aes geom_line geom_ribbon geom_histogram
#'   geom_density geom_col geom_hline labs theme_minimal after_stat
NULL

setGeneric("simulate",
           function(object, n, T_, steps, seed = NULL, ...)
             standardGeneric("simulate"))

setGeneric("loglik",
           function(object, log_returns, ...)
             standardGeneric("loglik"))

setGeneric("fit",
           function(object, log_returns, ...)
             standardGeneric("fit"))

setGeneric("jumpMoments",
           function(object) standardGeneric("jumpMoments"))

# -- Generic: simulate ------------------------------------------
#' Simulate Asset Price Paths
#'
#' @param object A \linkS4class{JumpDiffModel} object.
#' @param n      Integer. Number of simulated paths.
#' @param T_     Positive numeric. Time horizon in years.
#' @param steps  Positive integer. Number of time steps.
#' @param seed   Optional integer seed for reproducibility.
#' @param ...    Additional arguments passed to methods.
#' @return A \linkS4class{JDSimResult} object.
#' @export
setGeneric("simulate",
           function(object, n, T_, steps, seed = NULL, ...)
             standardGeneric("simulate"))

# -- Generic: loglik --------------------------------------------
#' Compute Negative Log-Likelihood
#'
#' @param object      A \linkS4class{JumpDiffModel} object.
#' @param log_returns Numeric vector of log asset returns.
#' @param ...         Passed to methods.
#' @return Scalar negative log-likelihood value.
#' @export
setGeneric("loglik",
           function(object, log_returns, ...)
             standardGeneric("loglik"))

# -- Generic: fit -----------------------------------------------
#' Fit Model to Log-Returns via MLE
#'
#' @param object      A \linkS4class{JumpDiffModel} object.
#' @param log_returns Numeric vector of observed log returns.
#' @param ...         Passed to \code{\link[stats]{optim}}.
#' @return A \linkS4class{JDFitResult} object.
#' @export
setGeneric("fit",
           function(object, log_returns, ...)
             standardGeneric("fit"))

# -- Generic: jumpMoments ---------------------------------------
#' Theoretical Moments of the Log-Return Distribution
#'
#' @param object A \linkS4class{JumpDiffModel} object.
#' @return Named numeric vector: mean, variance, skewness, kurtosis.
#' @export
setGeneric("jumpMoments",
           function(object) standardGeneric("jumpMoments"))

---
title: "Getting Started with JumpDiffSim"
author: "Kennedy Kayaki, Dohyun Oh, Ju Seong Hyeon, Lee Se Eun, Choi Jiwoo, Yuri Shin"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with JumpDiffSim}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  echo     = TRUE,
  message  = FALSE,
  warning  = FALSE,
  fig.width  = 6,
  fig.height = 4
)
```

## 1. Why Jump-Diffusion?

Standard Geometric Brownian Motion (GBM) assumes that log-returns are
normally distributed. Empirical equity and cryptocurrency returns, however,
exhibit well-documented departures from normality:

- **Leptokurtosis** (excess kurtosis > 0): more mass in the centre and tails
  than a Normal distribution predicts.
- **Negative skewness**: large negative moves are more common and more severe
  than large positive moves.

The **Merton (1976)** jump-diffusion model adds a compound Poisson process
to GBM, allowing the price to jump discontinuously at random times:

$$
dS_t = S_t \left[ (\mu - \lambda \bar{\mu}_J)\,dt +
       \sigma\,dW_t + dJ_t \right]
$$

where $J_t = \sum_{i=1}^{N(t)}(Y_i - 1)$, $N(t) \sim \text{Poisson}(\lambda t)$,
and $\log Y_i \sim \mathcal{N}(\mu_J, \sigma_J^2)$.

**JumpDiffSim** provides a clean, unified S4 interface for simulating paths
and calibrating parameters under this model, with all examples running
entirely offline.

---

## 2. Simulate Paths

```{r simulate, fig.width=6, fig.height=4}
library(JumpDiffSim)

# Create a MertonModel S4 object with default parameters
m <- MertonModel(
  mu      =  0.05,   # drift
  sigma   =  0.20,   # diffusion volatility
  lambda  =  1,      # average jumps per year
  mu_j    = -0.10,   # mean log-jump size
  sigma_j =  0.15    # std dev of log-jumps
)

# Display the model
show(m)

# Simulate 200 paths over 1 year with 252 daily steps
sim <- simulateMerton(m, n = 200, T_ = 1, steps = 252, seed = 42)

# Diagnostic plots
plts <- diagnosticPlots(sim)
print(plts$fan_chart)
print(plts$density)
```

The fan chart shows the 5th, 25th, 50th, 75th, and 95th percentile bands
of the 200 simulated paths. The density plot overlays the empirical
return histogram against a Normal curve, illustrating the heavy-tailed
character of Merton returns.

---

## 3. Fit Model to Data

```{r fit}
# Generate reproducible synthetic log-returns
# (all examples use jdSampleData() -- no internet required)
ret <- jdSampleData("merton", n = 500, seed = 42)

# Fit the Merton model via Maximum Likelihood Estimation
fit <- fitMerton(ret, verbose = FALSE)

# Parameter estimates and convergence
print(fit)

# 95% Wald confidence intervals
confint(fit)
```

The `fitMerton()` function uses L-BFGS-B optimisation and computes
Hessian-based standard errors via `numDeriv::hessian()`. The `converged`
slot confirms whether the optimiser reached a solution.

---

## 4. Interpreting Parameters

| Parameter | Meaning                             |
|-----------|-------------------------------------|
| mu        | Drift (expected continuous return)  |
| sigma     | Diffusion volatility                |
| lambda    | Average jumps per year              |
| mu_j      | Average log-size of each jump       |
| sigma_j   | Std deviation of jump sizes         |

A **negative** `mu_j` combined with a positive `lambda` implies that jumps
on average reduce the asset price — consistent with the crash-risk
interpretation of Merton (1976).

---

## 5. Theoretical Moments

```{r moments}
# Theoretical mean, variance, skewness, and excess kurtosis
# contributed by the jump component
jumpMoments(m)
```

Excess kurtosis greater than zero confirms that the Merton model generates
heavier tails than GBM, which is the key motivation for using a
jump-diffusion specification.

---

## References

- Merton, R.C. (1976). Option pricing when underlying stock returns are
  discontinuous. *Journal of Financial Economics*, 3(1-2), 125-144.
- Wickham, H. and Bryan, J. (2023). *R Packages* (2nd ed.). O'Reilly Media.
  <https://r-pkgs.org/>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{fit}
\alias{fit}
\title{Fit Model to Log-Returns via MLE}
\usage{
fit(object, log_returns, ...)
}
\arguments{
\item{object}{A \linkS4class{JumpDiffModel} object.}

\item{log_returns}{Numeric vector of observed log returns.}

\item{...}{Passed to \code{\link[stats]{optim}}.}
}
\value{
A \linkS4class{JDFitResult} object.
}
\description{
Fit Model to Log-Returns via MLE
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_fit.R
\name{mertonLogLik}
\alias{mertonLogLik}
\title{Negative Log-Likelihood for the Merton Jump-Diffusion Model}
\usage{
mertonLogLik(params, log_returns, dt = 1/252, N_max = 50L)
}
\arguments{
\item{params}{Named numeric vector:
\code{c(mu, sigma, lambda, mu_j, sigma_j)}.}

\item{log_returns}{Numeric vector of observed log asset returns.}

\item{dt}{Numeric. Length of each time step (default 1/252).}

\item{N_max}{Integer. Number of mixture terms (default 50).}
}
\value{
Scalar. Negative log-likelihood. Returns \code{Inf} for
invalid parameter values.
}
\description{
Evaluates the negative log-likelihood of observing \code{log_returns}
under the Merton (1976) model. The density is approximated by a
Gaussian mixture truncated at \code{N_max = 50} terms.
}
\examples{
ret <- jdSampleData("merton", n = 200, seed = 42)
p   <- c(mu = 0.05, sigma = 0.2, lambda = 1, mu_j = -0.1, sigma_j = 0.15)
mertonLogLik(p, ret)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_model.R
\docType{class}
\name{JumpDiffModel-class}
\alias{JumpDiffModel-class}
\title{JumpDiffModel: Virtual Base Class for Jump-Diffusion Models}
\description{
Abstract base class. Do not instantiate directly.
Concrete subclasses: \linkS4class{MertonModel}.
}
\section{Slots}{

\describe{
\item{\code{mu}}{Numeric. Drift parameter (annualised).}

\item{\code{sigma}}{Positive numeric. Diffusion volatility.}

\item{\code{lambda}}{Non-negative numeric. Jump intensity (jumps per year).}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_price.R
\name{priceEuropean}
\alias{priceEuropean}
\title{Price a European Option under the Merton Jump-Diffusion Model}
\usage{
priceEuropean(
  fit,
  S = 100,
  K = 100,
  r = 0.05,
  T_ = 1,
  type = "call",
  N_max = 50L
)
}
\arguments{
\item{fit}{A \linkS4class{JDFitResult} object from
\code{\link{fitMerton}}.}

\item{S}{Positive numeric. Current asset price.}

\item{K}{Positive numeric. Strike price.}

\item{r}{Numeric. Continuously compounded risk-free rate.}

\item{T_}{Positive numeric. Time to expiry in years.}

\item{type}{Character. Either \code{"call"} or \code{"put"}.}

\item{N_max}{Integer. Number of series terms (default 50).}
}
\value{
A named list with elements:
\item{price}{Merton jump-diffusion option price.}
\item{bs_price}{Black-Scholes benchmark price (no jumps).}
\item{jump_premium}{Difference between Merton and BS price.}
\item{params}{List of inputs and fitted parameters used.}
}
\description{
Computes the European call or put price under the Merton (1976)
jump-diffusion model using the analytic series expansion.
The price is expressed as a Poisson-weighted sum of
Black-Scholes prices with jump-adjusted drift and volatility.

The formula is:
\deqn{C = \sum_{n=0}^{N} \frac{e^{-\lambda' T}(\lambda' T)^n}{n!}
  \cdot BS(S, K, r_n, \sigma_n, T)}
where \eqn{\lambda' = \lambda(1 + \bar\mu_J)},
\eqn{r_n = r - \lambda\bar\mu_J + n\log(1+\bar\mu_J)/T},
and \eqn{\sigma_n^2 = \sigma^2 + n\sigma_J^2/T}.
}
\examples{
\donttest{
ret <- jdSampleData("merton", n = 500, seed = 42)
fit <- fitMerton(ret)
price <- priceEuropean(fit, S = 100, K = 100,
                       r = 0.05, T_ = 1, type = "call")
print(price)
}

}
\references{
Merton, R.C. (1976). Option pricing when underlying stock
returns are discontinuous.
\emph{Journal of Financial Economics}, 3(1-2), 125-144.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_model.R
\docType{class}
\name{JDSimResult-class}
\alias{JDSimResult-class}
\title{JDSimResult: Simulation Output Class}
\description{
JDSimResult: Simulation Output Class
}
\section{Slots}{

\describe{
\item{\code{paths}}{Matrix. Simulated price paths (n x steps+1).}

\item{\code{times}}{Numeric vector. Time grid.}

\item{\code{params}}{List. Model parameters used in simulation.}

\item{\code{model_type}}{Character. Model name.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_model.R, R/merton_fit.R
\docType{class}
\name{JDFitResult-class}
\alias{JDFitResult-class}
\alias{show,JDFitResult-method}
\alias{confint,JDFitResult-method}
\title{JDFitResult: MLE Estimation Output Class}
\usage{
\S4method{show}{JDFitResult}(object)

\S4method{confint}{JDFitResult}(object, parm, level = 0.95, ...)
}
\arguments{
\item{object}{A \linkS4class{JDFitResult} object.}

\item{parm}{Ignored (all parameters returned).}

\item{level}{Confidence level (default 0.95).}

\item{...}{Unused.}
}
\description{
JDFitResult: MLE Estimation Output Class
}
\section{Methods (by generic)}{
\itemize{
\item \code{confint(JDFitResult)}: Wald-type 95\\% confidence intervals.

}}
\section{Slots}{

\describe{
\item{\code{estimates}}{Named numeric. Parameter estimates.}

\item{\code{se}}{Named numeric. Standard errors.}

\item{\code{loglik}}{Numeric. Log-likelihood at optimum.}

\item{\code{converged}}{Logical. Did optimisation converge?}

\item{\code{model_type}}{Character. Model name.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_sim.R
\name{simulateMerton}
\alias{simulateMerton}
\title{Simulate Merton Jump-Diffusion Asset Paths}
\usage{
simulateMerton(object, n = 100, T_ = 1, steps = 252, S0 = 1, seed = NULL, ...)
}
\arguments{
\item{object}{A \linkS4class{MertonModel} object.}

\item{n}{Integer. Number of paths to simulate.}

\item{T_}{Positive numeric. Time horizon in years (default 1).}

\item{steps}{Positive integer. Number of time steps (default 252).}

\item{S0}{Numeric. Initial asset price (default 1).}

\item{seed}{Optional integer. Random seed for reproducibility.}

\item{...}{Unused.}
}
\value{
A \linkS4class{JDSimResult} object.
}
\description{
Generates \code{n} exact compound-Poisson asset price paths under the
Merton (1976) jump-diffusion model. Simulation is exact in the sense
that jump arrivals are drawn directly from the Poisson process rather
than approximated via Euler discretisation.
}
\examples{
m   <- MertonModel()
sim <- simulateMerton(m, n = 10, T_ = 1, steps = 252, seed = 42)
dim(sim@paths)   # should be 200 x 253

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_model.R
\name{MertonModel}
\alias{MertonModel}
\title{Create a MertonModel Object}
\usage{
MertonModel(mu = 0.05, sigma = 0.2, lambda = 1, mu_j = -0.1, sigma_j = 0.15)
}
\arguments{
\item{mu}{Numeric. Drift (default 0.05).}

\item{sigma}{Positive numeric. Diffusion volatility (default 0.20).}

\item{lambda}{Non-negative numeric. Jump intensity (default 1).}

\item{mu_j}{Numeric. Mean log-jump size (default -0.10).}

\item{sigma_j}{Positive numeric. Std dev of log-jumps (default 0.15).}
}
\value{
A validated \linkS4class{MertonModel} object.
}
\description{
Create a MertonModel Object
}
\examples{
m <- MertonModel(mu = 0.05, sigma = 0.20, lambda = 1,
                 mu_j = -0.10, sigma_j = 0.15)
show(m)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_sim.R
\name{diagnosticPlots}
\alias{diagnosticPlots}
\title{Diagnostic Plots for a JDSimResult Object}
\usage{
diagnosticPlots(object)
}
\arguments{
\item{object}{A \linkS4class{JDSimResult} object.}
}
\value{
A named list of three \code{ggplot} objects:
\itemize{
\item \code{fan_chart}: simulated path quantile fan chart.
\item \code{density}: histogram of log-returns with a normal overlay.
\item \code{acf_sq}: autocorrelation of squared log-returns with 95\\% bounds.
}
}
\description{
Creates three diagnostic plots for a \linkS4class{JDSimResult} object:
a simulated path fan chart, a histogram of log-returns with a normal
density overlay, and the autocorrelation function of squared log-returns.
}
\examples{
\donttest{
m    <- MertonModel()
sim  <- simulateMerton(m, n = 50, T_ = 1, steps = 100)
plts <- diagnosticPlots(sim)
print(plts$fan_chart)
print(plts$density)
print(plts$acf_sq)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_fit.R
\name{fitMerton}
\alias{fitMerton}
\title{Fit Merton Jump-Diffusion Model via MLE}
\usage{
fitMerton(
  log_returns,
  start = c(mu = 0.05, sigma = 0.2, lambda = 1, mu_j = -0.1, sigma_j = 0.15),
  dt = 1/252,
  N_max = 50L,
  verbose = FALSE
)
}
\arguments{
\item{log_returns}{Numeric vector of observed log asset returns.}

\item{start}{Named numeric vector of starting values.
Defaults to \code{c(mu=0.05, sigma=0.20, lambda=1,
  mu_j=-0.10, sigma_j=0.15)}.}

\item{dt}{Numeric. Time step length (default 1/252).}

\item{N_max}{Integer. Mixture truncation (default 50).}

\item{verbose}{Logical. Print progress? (default FALSE).}
}
\value{
A \linkS4class{JDFitResult} object.
}
\description{
Maximises the log-likelihood of the Merton (1976) model using
L-BFGS-B optimisation. Returns a \linkS4class{JDFitResult} object
containing parameter estimates, standard errors, and convergence info.
}
\examples{
\donttest{
ret <- jdSampleData("merton", n = 500, seed = 42)
fit <- fitMerton(ret, verbose = TRUE)
print(fit)
confint(fit)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{simulate}
\alias{simulate}
\title{Simulate Asset Price Paths}
\usage{
simulate(object, n, T_, steps, seed = NULL, ...)
}
\arguments{
\item{object}{A \linkS4class{JumpDiffModel} object.}

\item{n}{Integer. Number of simulated paths.}

\item{T_}{Positive numeric. Time horizon in years.}

\item{steps}{Positive integer. Number of time steps.}

\item{seed}{Optional integer seed for reproducibility.}

\item{...}{Additional arguments passed to methods.}
}
\value{
A \linkS4class{JDSimResult} object.
}
\description{
Simulate Asset Price Paths
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_model.R
\docType{class}
\name{MertonModel-class}
\alias{MertonModel-class}
\alias{show,MertonModel-method}
\title{MertonModel: Merton (1976) Jump-Diffusion Model}
\usage{
\S4method{show}{MertonModel}(object)
}
\arguments{
\item{object}{A \linkS4class{MertonModel} object.}
}
\description{
Extends \linkS4class{JumpDiffModel} with log-normal jump parameters.
}
\section{Slots}{

\describe{
\item{\code{mu_j}}{Numeric. Mean log-jump size.}

\item{\code{sigma_j}}{Positive numeric. Std dev of log-jump sizes.}
}}

\references{
Merton, R.C. (1976). Option pricing when underlying stock returns are
discontinuous. \emph{Journal of Financial Economics}, 3(1-2), 125-144.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{jumpMoments,MertonModel-method}
\alias{jumpMoments,MertonModel-method}
\title{Theoretical Moments of the Merton Jump-Diffusion Log-Return}
\usage{
\S4method{jumpMoments}{MertonModel}(object)
}
\arguments{
\item{object}{A \linkS4class{MertonModel} object.}
}
\value{
Named numeric vector with elements
\code{mean}, \code{variance}, \code{skewness}, \code{kurtosis}.
}
\description{
Returns the theoretical mean, variance, skewness, and excess kurtosis
of the log-return distribution under the Merton (1976) model.
}
\examples{
m <- MertonModel(mu = 0.05, sigma = 0.20, lambda = 1,
                 mu_j = -0.10, sigma_j = 0.15)
jumpMoments(m)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{loglik}
\alias{loglik}
\title{Compute Negative Log-Likelihood}
\usage{
loglik(object, log_returns, ...)
}
\arguments{
\item{object}{A \linkS4class{JumpDiffModel} object.}

\item{log_returns}{Numeric vector of log asset returns.}

\item{...}{Passed to methods.}
}
\value{
Scalar negative log-likelihood value.
}
\description{
Compute Negative Log-Likelihood
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merton_sim.R
\name{jdSampleData}
\alias{jdSampleData}
\title{Generate Synthetic Log-Returns from a Jump-Diffusion Model}
\usage{
jdSampleData(
  model = "merton",
  n = 500,
  mu = 0.05,
  sigma = 0.2,
  lambda = 1,
  mu_j = -0.1,
  sigma_j = 0.15,
  dt = 1/252,
  seed = 42L
)
}
\arguments{
\item{model}{Character. Model type: \code{"merton"} (default).}

\item{n}{Integer. Number of log-returns to generate (default 500).}

\item{mu}{Numeric. Drift (default 0.05).}

\item{sigma}{Positive numeric. Diffusion vol (default 0.20).}

\item{lambda}{Non-negative numeric. Jump intensity (default 1).}

\item{mu_j}{Numeric. Mean log-jump (default -0.10).}

\item{sigma_j}{Positive numeric. Std dev of log-jumps (default 0.15).}

\item{dt}{Numeric. Time step (default 1/252).}

\item{seed}{Integer. Random seed (default 42).}
}
\value{
Numeric vector of \code{n} log-returns.
}
\description{
Convenience function for generating reproducible synthetic log-returns
for use in examples, tests, and vignettes. All package examples and
tests use this function to avoid any dependency on live market data.
}
\examples{
ret <- jdSampleData("merton", n = 200, seed = 42)
head(ret)
hist(ret, breaks = 40, main = "Synthetic Merton Returns")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{jumpMoments}
\alias{jumpMoments}
\title{Theoretical Moments of the Log-Return Distribution}
\usage{
jumpMoments(object)
}
\arguments{
\item{object}{A \linkS4class{JumpDiffModel} object.}
}
\value{
Named numeric vector: mean, variance, skewness, kurtosis.
}
\description{
Theoretical Moments of the Log-Return Distribution
}
